/** @file artnet.h
 *  @brief This module contains continuous tasks for artnet node communications.
 *
 * This is the header file for the definition of services to allow managing thread acting as artnet
 * node.
 */
#if !defined(__ARTNET_H__)
#define __ARTNET_H__

/***************************************************************************************************
 * Includes
 **************************************************************************************************/
#include <stdint.h>

/***************************************************************************************************
 * Preprocessor Constants and Macros
 **************************************************************************************************/

/***************************************************************************************************
 * Type and Contant Definitions
 **************************************************************************************************/

/***************************************************************************************************
 * Global Variables
 **************************************************************************************************/

/***************************************************************************************************
 * External Function Prototypes
 **************************************************************************************************/

/**
 * @brief Start artnet node module
 **/
void artnet_start();

/**
 * @brief Stop artnet node module
 */
void artnet_stop();

/**
 * @brief Get last DMX data received for the specified universe
 *
 * @param[in]   univerve The universe to get data from
 *
 * @param[out]  dmxData  The pointer to the DMX data array
 */
int artnet_get_dmx_data(unsigned int univerve, uint8_t *dmxData[]);

#endif /* __ARTNET_H__ */
